@@ -287,16 +287,6 @@ on_sighup_cb (gpointer user_data)
         return TRUE;
 }
 
-static gboolean
-on_sigusr1_cb (gpointer user_data)
-{
-        g_debug ("Got USR1 signal");
-
-        gdm_log_toggle_debug ();
-
-        return TRUE;
-}
-
 static gboolean
 is_debug_set (void)
 {
@@ -305,6 +295,19 @@ is_debug_set (void)
         return debug;
 }
 
+/* SIGUSR1 is used by the X server to tell us that we're ready, so
+ * block it. We'll unblock it in the worker thread in gdm-server.c
+ */
+static void
+block_sigusr1 (void)
+{
+        sigset_t mask;
+
+        sigemptyset (&mask);
+        sigaddset (&mask, SIGUSR1);
+        sigprocmask (SIG_BLOCK, &mask, NULL);
+}
+
 int
 main (int    argc,
       char **argv)
@@ -325,6 +328,8 @@ main (int    argc,
                 { NULL }
         };
 
+        block_sigusr1 ();
+
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
         textdomain (GETTEXT_PACKAGE);
         setlocale (LC_ALL, "");
@@ -400,7 +405,6 @@ main (int    argc,
         g_unix_signal_add (SIGTERM, on_shutdown_signal_cb, main_loop);
         g_unix_signal_add (SIGINT, on_shutdown_signal_cb, main_loop);
         g_unix_signal_add (SIGHUP, on_sighup_cb, NULL);
-        g_unix_signal_add (SIGUSR1, on_sigusr1_cb, NULL);
 
         if (do_timed_exit) {
                 g_timeout_add_seconds (30, (GSourceFunc) timed_exit_cb, main_loop);
@@ -444,8 +448,6 @@ on_name_acquired (GDBusConnection *bus,
 
         g_debug ("Successfully connected to D-Bus");
 
-        gdm_manager_start (manager);
-
         show_local_greeter = TRUE;
         gdm_settings_direct_get_boolean (GDM_KEY_SHOW_LOCAL_GREETER, &show_local_greeter);
         gdm_manager_set_show_local_greeter (manager, show_local_greeter);
@@ -453,6 +455,8 @@ on_name_acquired (GDBusConnection *bus,
         xdmcp_enabled = FALSE;
         gdm_settings_direct_get_boolean (GDM_KEY_XDMCP_ENABLE, &xdmcp_enabled);
         gdm_manager_set_xdmcp_enabled (manager, xdmcp_enabled);
+
+        gdm_manager_start (manager);
 }
 
 static void
